/*
 *   ScenarLang a domain specific language that attempts to make it even easier
 *   to write unit tests with mockito
 *    
 *   Copyright (C) 2012  X. Gillard 
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package xgi.ut.dsl

import scala.util.parsing.combinator.JavaTokenParsers

/**
 * This contains the grammar of an internal DSL meant to simplify our lives 
 * when it comes to configuring the parsing/conversion of some given types
 * these types are represented as strings in the "program" text
 */
trait ConversionConfigParsers extends JavaTokenParsers {
  import ConversionConfigParsers._
  /* parses a conversion configuration */
  def conversionconf:Parser[ConversionConfig]="parse"~>stringLiteral~"with"~stringLiteral ^^ {
    case typeName~"with"~conversionClass => ConversionConfig(typeName.drop(1).dropRight(1), conversionClass.drop(1).dropRight(1))
  }
  /* returns a conversion config that is maybe a syntax error*/
  def parseConversion(text:String):ConversionConfig = parseAll(conversionconf, text) match {
    case Success(x, _) => x
    case NoSuccess(mesg, _) => ConversionConfigSyntaxError(mesg)
  }
}

/* the companion object */
object ConversionConfigParsers {
  sealed case class ConversionConfig(typeName:String, parserClass:String)
  
  sealed case class ConversionConfigSyntaxError(mesg:String) extends ConversionConfig("SYNTAX ERROR", "SYNTAX ERROR")
}