/*
 *   ScenarLang a domain specific language that attempts to make it even easier
 *   to write unit tests with mockito
 *    
 *   Copyright (C) 2012  X. Gillard 
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package xgi.ut.dsl.interpreters

import xgi.ut.dsl.JsonParsers.JsonArray
import xgi.ut.dsl.JsonParsers.JsonBasicType
import xgi.ut.dsl.JsonParsers.JsonLiteral
import xgi.ut.dsl.JsonParsers.JsonProperty
import xgi.ut.dsl.JsonParsers.JsonStructure
import xgi.ut.dsl.JsonParsers.JsonSyntaxError

/**
 * Utility trait to pretty print a json literal (into something that is actually json parseable)
 */
trait PrettyPrinter {
  /**
   * returns the jsonliteral in a pretty printed string
   */
  def prettyPrint(json:JsonLiteral)(implicit depth:Int=0):String = {
    val buf = new StringBuilder
    val indent = "  "*depth
    
    json match {
    case JsonBasicType(value, _) => {
      val printVal = value match {
        case x:String => "\""+x+"\""
        case _ => value
      }
      buf append indent append printVal toString
    }
    case JsonArray(elems) => {
      buf append "["
      buf append ((for(e <- elems) yield {prettyPrint(e)(depth+1)}).mkString("\n",", \n", "\n"))
      buf append indent append "]"
      buf toString
    }
    case JsonStructure(props, _) => {
      buf append "{"
      buf append ((for(e <- props) yield {prettyPrint(e)(depth+1)}).mkString("\n",", \n", "\n"))
      buf append indent append "}"
      buf toString
    }
    case JsonProperty(k, v) => {
      buf append indent append "\"" append k append "\" : " append prettyPrint(v)(depth+1).trim toString 
    }
    case JsonSyntaxError(mesg) => Console.err.println(mesg); mesg
    //case other => other toString
    }
  }
}