/*
 *   ScenarLang a domain specific language that attempts to make it even easier
 *   to write unit tests with mockito
 *    
 *   Copyright (C) 2012  X. Gillard 
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package xgi.ut.dsl.interpreters.ext

import xgi.ut.dsl.ScenarParsers
import xgi.ut.dsl.interpreters.ScenarioInterpreter
import java.io.File
import scala.io.Source
import scala.collection.mutable.StringBuilder
import scala.collection.immutable.VectorBuilder

/**
 * this class will let you interpret any given scenario
 */
class ScenarioInterpretation(val context:Any, script:File) extends ScenarParsers with ScenarioInterpreter {
  /** a reg ex that matches blank line */
  val IsBlank = "(\\s*)".r
  /** a reg ex that matches line comment */
  val IsLineComment = "(//)(.*)".r 
  
  val validLines = Source.fromFile(script).getLines withFilter{ _ match {
      case IsBlank(y) => false
      case IsLineComment(y, _) => false
      case _ => true
    }
  }
  
  /** this defines whether one line continues the previous */
  val IsContinuation = "~>(.*)".r
  val mutableText = new VectorBuilder[StringBuilder]
  validLines foreach { _ match {
      case IsContinuation(text) => mutableText.result.last append text
      case other => mutableText += new StringBuilder(other)
    }
  }
  mutableText.result foreach {x=>parseScenario(x.result) foreach interpret}
}